#! /usr/bin/perl
#  Copyright 2001-2011 Leslie Richardson

#  This file is part of Open Admin for Schools.

# ======= Subject Attendance Report ======= 
# Get Start date of current term. Calculate all student's attendance
# per subject (and grand total). Display descending list by all students
#  per class or per total. Data comes first from attendance linked to other
#  tables. Attend is definitive, not eval (which may change).

my %lex = ('Attendance Report' => 'Attendance Report',
	   'Attendance' => 'Attendance',
	   'Main' => 'Main',
	   'Not Found' => 'Not Found',
	   'Error' => 'Error',
	   'Terms' => 'Terms',
	   'Continue' => 'Continue',
	   'Red' => 'Red',
	   'Withdrawn' => 'Withdrawn',
	   'Absence' => 'Absence',

	   );

use DBI;
use CGI;


# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset );

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


# Get Additional Comment values from configuration system, in order to skip...
my $sth = $dbh->prepare("select datavalue from conf_system 
  where dataname = 'r_AdditionalComments'");
$sth->execute;
my $datavalue = $sth->fetchrow;
eval $datavalue;
if ( $@ ) {
    print "$lex{Error}: $@<br>\n";
    die "$lex{Error}: $@\n";
}


my $title = "$lex{Subject} $lex{'Attendance Report'}";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype</head><body>\n";


print "[ <a href=\"$homepage\">$lex{Main}</a> | \n";
print "<a href=\"$attpage\">$lex{Attendance}</a> ]\n";

print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

}


my ($sterm, $eterm) = split('-', $arr{term});

print "<p>$lex{Terms}: $sterm &ndash; $eterm \n";
print "&nbsp;&nbsp;<span style=\"color:red\">$lex{Red}=$lex{Withdrawn}</span></p>\n";


# First find subjects in this term or term block.
my (@subjects, %subjects);
my $sth = $dbh->prepare("select subjsec, description from subject where
 startrptperiod = ? and endrptperiod = ? order by description");
$sth->execute($sterm, $eterm);
if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
while ( my ( $subjsec, $desc) = $sth->fetchrow ) {

    my ($subjcode, $dud ) = split(/-/, $subjsec);
    # skip if member of %r_AdditionalComments hash - NEW
    if ( $r_AdditionalComments{$subjsec} or $r_AdditionalComments{$subjcode} ) {
	next; 
    }

    push @subjects, $subjsec; # @subjects gives us sort order
    $subjects{$subjsec} = $desc;
}


# Setup for each Student for this subject
my $sth1 = $dbh->prepare("select distinct a.studentid, 
  s.lastname, s.firstname from attend a
  left outer join student s on s.studnum = a.studentid
  where a.subjsec = ? order by s.lastname, s.firstname");

# Setup  for Student/Subject Attendance getting reasons count.
my $sth2 = $dbh->prepare("select distinct reason, count(reason) from attend 
  where studentid = ? and subjsec = ? group by reason");

my $sth3 = $dbh->prepare("select distinct reason from attend  where subjsec = ?");
my $sth4 = $dbh->prepare("select count(*) from eval where subjcode = ?");

my $firstflag = 1;

# Start Table
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";

# Now loop through all subjects, printing students and number missed.
foreach my $subjsec ( @subjects ){

    my $description = $subjects{$subjsec};

    # Check for enrollments.
    $sth4->execute( $subjsec );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $enrolcount = $sth4->fetchrow;
    if ( not $enrolcount ) { next; } # skip ths subject.


    # Find reasons for this subject. (could do global, but may be smaller)
    $sth3->execute( $subjsec );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my @reasons;
    while ( my $reason = $sth3->fetchrow ) {
	push @reasons,$reason;
    }


    if (not $firstflag) {
	print "</table><p>[ <a href=\"$attpage\">Attendance Page</a> ]</p>\n";
    } else { $firstflag = 0; }


    print "<h2>$description ($subjsec)</h2>\n";

    if ( not @reasons ) { 
	print "<p>$lex{Absence} $lex{'Not Found'}</p>\n";
	next; # subject
    }


    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>$lex{Student}</th>\n";
    foreach my $rsn (@reasons){
	print "<th>$rsn</th>";
    }
    print "</tr>\n";

    # Get Studnum, Lastname, Firstname for this subject.
    $sth1->execute( $subjsec );
    if ($DBI::errstr){ print $DBI::errstr; die;}
    my $studary = $sth1->fetchall_arrayref;

    foreach my $studref (@$studary){ # Loop through all students.
	my $studnum = $$studref[0];
	my $lastname = $$studref[1];
	my $firstname = $$studref[2];

	if (not $lastname){ # look in withdrawn students
	    my $sth3 = $dbh->prepare("select lastname, firstname from studentwd
             where studnum = '$studnum'");
	    $sth3->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    ($lastname,$firstname) = $sth3->fetchrow;
	    if (not $lastname){ $lastname = $lex{'Not Found'};}
	    $lastname = "<span style=\"color:red;\">$lastname</span>";
	    $firstname = "<span style=\"color:red;\">$firstname</span>";
	}

	# Get reasons and totals for this student
	$sth2->execute($studnum,$subjsec);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errtstr;}
	my $rsnary = $sth2->fetchall_arrayref;
	my %reason;
	foreach my $rowref (@$rsnary){ 
	    $reason{$$rowref[0]} = $$rowref[1];
	}

	# Now loop through reasons (NOT from admin.conf, but attend array)
	print "<tr><td>$lastname, $firstname ($studnum)</td>\n";
	foreach my $rsn (@reasons){
	    print "<td>$reason{$rsn}</td>\n";
	}
	print "</tr>\n";
    }

}

print"</table></body></html>\n";



#----------------
sub showStartPage {
#----------------

    # Find all of the term patterns.
    my $sth = $dbh->prepare("select distinct startrptperiod, endrptperiod from subject
     order by startrptperiod, endrptperiod");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table border=\"0\" cellpadding=\"3\" cellspacing=\"0\">\n";

    # Term Select
    print "<tr><td class=\"bra\">$lex{Terms}</td><td class=\"la\">";
    print "<select name=\"term\"><option></option>\n";
    my %terms = ();
    while ( my ( $startterm, $endterm ) = $sth->fetchrow ) {
	    print "<option value=\"$startterm-$endterm\">";
	    print "$startterm-$endterm</option>\n";
	    # $terms{$startterm} = 1;
	    # $terms{$endterm} = 1;
    }

    # Now do single terms also.
#    foreach my $t ( sort {$a <=> $b} keys %terms ) {
#	    print "<option value=\"$t\">$t</option>\n";
#    }
    print "</select></td></tr>\n";


=head

    # Sort Order
    print "<tr><td class=\"bla\">$lex{'Sort by'}</td><td>";
    print "<select name=\"sort\"><option value=\"name\">$lex{Name}</option>\n";
    print "<option value=\"grade\">$lex{Grade}</option>\n";
    print "</select></td></tr>\n";
=cut

    # Continue
    print "<tr><td colspan=\"2\" class=\"cn\"><input type=\"submit\" value=\"$lex{Continue}\">\n";
    print "</td></tr>\n";

    print "</table></form></body></html>\n";

    exit;

}
