#!/usr/bin/perl
#  Copyright 2001-2010 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Eoy' => 'Eoy',
	   'Error' => 'Error',
	   'Add Ldap User' => 'Add Ldap User',
	   'Name' => 'Name',
	   'Select' => 'Select',
	   'Continue' => 'Continue',
	   'Present' => 'Present',

	   );

my $self = 'addstaffldap.pl';

# Should be stored externally soon
my $basedn = 'dc=jp2,dc=loccsd,dc=ca';
my $servername = 'test.loccsd.ca'; # address of the LDAP server to manage.
my $group = 'ou=staff';

use DBI;
use CGI;
use Net::LDAP;
use Net::LDAP::Entry;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

my $ldap = Net::LDAP->new( $servername ) or die "$@\n";
my $mesg = $ldap->bind('cn=admin,dc=jp2,dc=loccsd,dc=ca',
		       password => 'password'
		       );

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


# Print Page Header
print "$doctype\n<html><head><title>\n";
print $lex{'Add Ldap User'};
print "</title><link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";

print "$chartype\n</head><body>[ <a href=\"$homepage\">";
print $lex{Main}. "</a> | <a href=\"$eoypage\">";
print $lex{Eoy}. "</a> ]<center><h1>";
# Update this depending on final script locations.

print '<h1>'. $lex{'Add Ldap User'}. "</h1>\n";

if ( not $arr{page} ) {
    showStartPage();
} else {
    delete $arr{page};
    writeLdapRecord();
}


#----------------
sub showStartPage {
#----------------

    # Search LDAP for staff uid.
    my $searchbase = 'ou=staff,'. $basedn; 
    $mesg = $ldap->search( # perform a search
			   base => $searchbase,
			   attrs => ['uid'],
			   filter => "(& (ou=staff) )",
			   );

    # check for errors
    $mesg->code && die $mesg->error;

    # Get list of ldap staff.
    my %ldapstaff = ();
    foreach my $entry ($mesg->entries) { 
	my $uid = $entry->get_value( 'uid' );
	$ldapstaff{$uid} = 1;
    }

    $mesg = $ldap->unbind;   # take down session


    my $sth = $dbh->prepare("select distinct userid, lastname, firstname from staff
       order by lastname, firstname");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    # Start Form.
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<table cellpadding=\"3\" border=\"0\" cellspacing=\"0\">\n";

    print "<tr><th>". $lex{Name}. "</th><th>". $lex{Select}. "</th></tr>\n";

    while ( my ( $userid, $lastname, $firstname ) = $sth->fetchrow ) {

	print "<tr><td><b>$lastname</b>, $firstname</td>\n<td>";
	if ( $ldapstaff{$userid} ) { # already found
	    print $lex{Present};
	} else {
	    print "<input type=\"checkbox\" name=\"$userid\" value=\"1\">";
	}
	print "</td></tr>\n";

    }

    # Print Submit Row
    print "<tr><td colspan=\"2\" align=\"center\">";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";

    print "</table></form></center></body></html>\n";

    exit;

} # end of showStartPage


#------------------
sub writeLdapRecord {
#------------------

    foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    print "<br>\n";

    my $sth = $dbh->prepare("select lastname, firstname, ldap_uid, 
       ldap_uidnumber, ldap_gidnumber, password from staff where userid = ?");

    foreach my $userid ( sort keys %arr ) {

	my $entry = Net::LDAP::Entry->new;

	# Get User Information.
	$sth->execute( $userid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname, $ldapuid, $uidnumber, $gidnumber, $password ) 
	    = $sth->fetchrow;

	# Encrypt Password
	my $newpassword = `/usr/local/sbin/slappasswd -h {md5} -s $password`;
	chomp $newpassword;

	my $dn = 'uid='. "$ldapuid,$group,$basedn";
	$entry->dn($dn);

	print "$firstname $lastname ($userid)\n";

#	$entry->delete();

#=pod
	# Add the Entry
	$entry->add(
	     ou => 'staff',
	     o => 'jp2.loccsd.ca',
	     cn => "$firstname $lastname",
	     objectClass => 'person',
	     objectClass => 'organizationalPerson',
	     objectClass => 'inetOrgPerson',
	     objectClass => 'posixAccount',
	     objectClass => 'shadowAccount',
	     objectClass => 'top',
	     sn => $lastname,
	     givenName => $firstname,
	     uid => $ldapuid,
	     uidNumber => $uidnumber,
	     gidNumber => $gidnumber,
	     gecos => "$firstname $lastname",
	     loginShell => '/bin/bash',
	     homeDirectory => "/home/$ldapuid",
	     userPassword => "$newpassword"

	     );

#=cut

	$entry->update( $ldap ); # update this entry.

	print " <b>Added</b><br>\n";

    } # end user add loop




} # end of writeLdapRecord
