#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Attendance Records' => 'Attendance Records',
	   'Main' => 'Main',
	   'Homeroom' => 'Homeroom',
	   'Please Log In' => 'Please Log In',
	   'Student' => 'Student',
	   'Date' => 'Date',
	   'Period' => 'Period',
	   'Reason' => 'Reason',
	   'Edit' => 'Edit',
	   'Delete' => 'Delete',
	   'Duration' => 'Duration',
	   'Error' => 'Error',
	   'Homeroom' => 'Homeroom',
	   );


use DBI;
use CGI qw/escape unescape/;
use CGI::Session;

eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my $dsn = "DBI:mysql:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my $q = CGI->new;
my %arr = $q->Vars; # Get passed values


my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Setup Session
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

# Get Session Values (a defined userid means it was passed)
my $userid;
if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header( -charset, $charset );
    printHtmlHeader();
    print "<h1>". $lex{'Please Log In'}. "</h1>\n";
    print "</body></html>\n";
    exit;
}
# Ok, we have a login. Values below we have in environment.

$userid = $session->param('userid');
my $duration = $session->param('duration');
if (not ($duration =~ m/\+/) ) {
    $duration = '+'. $duration. 'm';
}

$session->expire('logged_in',$duration); # duration is already in +20m format;
print $session->header( -charset, $charset );


# Get the users homeroom(s)
$sth = $dbh->prepare("select field_value from staff as s, staff_multi as sm
  where s.userid = sm.userid and s.userid = ? and field_name = 'homeroom'");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
while ( my $hr = $sth->fetchrow ) {
    push @homerooms, $hr;
}


# Print top of page
printHtmlHeader();

print "$userid &ndash; $lex{Duration}: $duration\n";
print "<h1>$lex{Edit}/$lex{Delete} $lex{'Attendance Records'}</h1>\n";
print "<h3>$currdate ";
for my $hr ( @homerooms ) { print "| <a href=\"#$hr\">$lex{Homeroom} $hr</a>\n"; };
print "]</h3>\n";


# Now select the student attendance records for this room
my $fieldlist = "a.attid, a.studentid, a.absdate, a.reason, ".
 "a.period, s.lastname, s.firstname";

my $sth = $dbh->prepare("select $fieldlist from attend as a, studentall as s 
 where a.studentid = s.studnum and s.homeroom = ? 
 order by a.absdate desc, s.lastname, s.firstname");

# Do all homerooms
for my $hr ( @homerooms ) {

    print "<a name=\"$hr\"></a>\n";
    print "<div style=\"font-size:150%;font-weight:bold;\">". $lex{'Homeroom'};
    print " <b>$hr</b></div>";

    print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";
    print '<tr><th>'. $lex{Student}. '</th><th>'. $lex{Date}. '</th><th>';
    print $lex{Period}. ' &ndash; '. $lex{Reason}. '</th><th>'. $lex{Edit};
    print '</th><th>'. $lex{Delete}. "</th></tr>\n";

    $sth->execute( $hr );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    while ( my ( $attid, $studentid, $absdate, $reason, $period, $lastname,
		 $firstname) = $sth->fetchrow ) {

	print "<tr><td><b>$lastname</b> $firstname</td><td>$absdate</td>";
	print "<td>$period - $reason</td>\n";

	print "<td><form action=\"tatted.pl\" method=\"post\">\n";
	print "<input type=\"hidden\" name=\"id\" value=\"$attid\">\n";
	print "<input type=\"submit\"  value=\"". $lex{Edit}. "\">\n";
	print "</form></td>\n";

	print "<td><form action=\"tattdel.pl\" method=\"post\">\n";
	print "<input type=\"hidden\" name=\"id\" value=\"$attid\">\n";
	print "<input type=\"submit\"  value=\"". $lex{Delete}. "\">\n";
	print "</form></td></tr>\n";

    }

    print "</table><p></p>\n\n";

}

print "</body></html>\n";




#------------------
sub printHtmlHeader {
#------------------

    print "$doctype\n<html><head><title>$lex{Edit}/$lex{Delete} $lex{'Attendance Records'}</title>\n";
    print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">\n";
    print qq{$chartype\n</head><body style="padding:1em;">\n};

    print "[ <a href=\"$tchpage\">$lex{Main}</a> ]\n";

}
